function [class,acc,Z,W,T1]=regec(train,train_l,test,test_l,sigma,delta1,delta2)
% Authors:
% Mario Guarracino, Claudio Cifarelli, Onur Seref, Panos Pardalos
% version 2.0
% 2005
%
% function [class,acc,Z,W,T1,time]=regec(train,train_l,test,test_l,sigma,delta1,delta2)
% Regec2 classifies two sets of points after a training phase. It implements
% a classification algorithm based on generalized eigenvalue problems.
%
% Input variables
% train:    (input)  contains all training points;
%
% train_1:  (input)  vector of classification labels.
%
% test:     (input)  contains all testing points;
%
% test_1:   (input)  vector of classification labels.
%
% sigma:    (input)  gaussian kernel parameter (denominator). 
% 
% delta1:    (input)  regularization parameter for the first hyperplane
%                    (usually 1e-03 - 1e-04 - 1e-05 )
% 
% delta2:    (input)  regularization parameter for the second hyperplane
%                    (usually same value of delta1 )
% 
% Output variables
% class:    (output) vector of all evaluated classification labels.
%
% acc:      (output) accuracy of computed classification.
%
% Z:        (output) contains distance from first and last eigenvector and  
%                    classification labels for all test points. 
% W:        (output) contains the coefficients of the two planes
%
% load training data in A and B

A=train(train_l==1,:);
B=train(train_l==-1,:);
C=[A;B];

% Build left and right matrices to be used in generalize eigenvalue
% problems solution using a gaussian kernel
g=[kerneljep(A,C,sigma), -ones(size(A,1),1)];
h=[kerneljep(B,C,sigma), -ones(size(B,1),1)];
G1=g'*g;
H1=h'*h;
T=diag(diag(H1),0);
U=diag(diag(G1),0);

% Build planes
G = G1 + delta1*T;
H = H1 + delta2*U;

[V1,D1] = eig(G,H);
[m1,imin1] = min( diag(abs(D1)) );
[m2,imax2] = max( diag(abs(D1)) );
W=[V1(:,imin1) V1(:,imax2)] ;
aa=sort(diag(abs(D1)));
sizeD1=size(D1,1);
T1=[ aa(1) aa(sizeD1) abs(aa(1) - aa(2))/abs(aa(1)), abs(aa(sizeD1)-aa(sizeD1-1))/abs(aa(sizeD1))];

% Classify test points
n=size(C,1);
Z=[];


z1=(abs(kerneljep(test,C,sigma)*V1(1:n,imin1)-V1(n+1,imin1)).^2)/(norm(V1(1:n,imin1))^2);
z2=(abs(kerneljep(test,C,sigma)*V1(1:n,imax2)-V1(n+1,imax2)).^2)/(norm(V1(1:n,imax2))^2);

   Z=[z1 z2 test_l];

tutti=0;
totti=0;

% Evaluate accuracy
class=sign(-(Z(:,1)-Z(:,2)));
if(sum(class==1)==0) 
    tutti=1;
end
if(sum(class==-1)==0) 
    totti=1;
end
    
acc=sum((class-test_l)==0)/size(class,1);

